import {TypedRecord, makeTypedFactory} from "typed-immutable-record";

import {
    FormDefinition,
    FormMetadata,
    Section,
    SectionMetadata,
    FormInput,
    DropdownOption
} from './form.server.models'

export interface FormDefinitionRecord extends TypedRecord<FormDefinitionRecord>, FormDefinition {}
export interface FormMetadataRecord extends TypedRecord<FormMetadataRecord>, FormMetadata {}
export interface SectionRecord extends TypedRecord<SectionRecord>, Section {}
export interface SectionMetadataRecord extends TypedRecord<SectionMetadataRecord>, SectionMetadata {}
export interface FormInputRecord extends TypedRecord<FormInputRecord>, FormInput {}
export interface DropdownOptionRecord extends TypedRecord<DropdownOptionRecord>, DropdownOption {}

export class ServerFactoryImpl {
    formMetadataRecord = makeTypedFactory<FormMetadata, FormMetadataRecord>({
        formId: null,
        instanceId: null,
        title: null
    });

    formDefinitionRecord = makeTypedFactory<FormDefinition, FormDefinitionRecord>({
        meta: this.formMetadataRecord(),
        sections: []
    });

    sectionMetadataRecord = makeTypedFactory<SectionMetadata, SectionMetadataRecord>({
        title: null,
        sectionId: null,
        layout: 'simple'
    });

    sectionRecord = makeTypedFactory<Section, SectionRecord>({
        meta: this.sectionMetadataRecord(),
        layout: []
    });

    formInputRecord = makeTypedFactory<FormInput, FormInputRecord>({
        Name: 'unknown',
        Type: 'unknown',
        Label: '',
        Required: false,
        Options: null
    });

    dropdownOptionRecord = makeTypedFactory<DropdownOption, DropdownOptionRecord>({
        Label: '',
        Value: ''
    });
}

export const ServerFactory = new ServerFactoryImpl();